;;; **********************************************************************
;;; Copyright (C) 2005 Heinrich Taube, <taube (at) uiuc (dot) edu>
;;;
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the Lisp Lesser Gnu Public License.
;;; See http://www.cliki.net/LLGPL for the text of this agreement.
;;; **********************************************************************

;;; $Name: rel-2_0_0 $
;;; $Revision: 1.2 $
;;; $Date: 2006/04/27 17:53:13 $


(in-package :portmidi)

(defparameter *libpmrecv-exists* nil)

(let* ((ext #+win32 "dll"
	    #+(or darwin macos macosx) "dylib"
	    #+(or linux (and clisp unix (not macos))) "so")
       (lib (make-pathname :name "libpmrecv" :type ext
			   :defaults (truename *load-pathname*))))
  (if (probe-file lib)
      (progn
	(cffi:load-foreign-library lib)
	(setf *libpmrecv-exists* t))))

;;(cffi:load-foreign-library "/usr/local/lisp/portmidi/libpmrecv.dylib")


(if *libpmrecv-exists* 
    (progn
      (cffi::defcfun ("pm_set_input" pm-set-input)
	  :int
	(pmstream :pointer)
	(id :int)
	(bufsize :int))

      (cffi::defcfun ("pm_recv_start" pm-recv-start)
	  :int
	(priority :int)
	(resolution :int)
	(recv-callback :pointer))
      
      (cffi::defcfun ("pm_recv_stop" pm-recv-stop)
	  :int)
      
      (cffi::defcfun ("pm_recv_state" pm-recv-state)
	  :int)
      
      (cffi::defcfun ("pm_priority_min" pm-priority-min)
	  :int)
      
      (cffi::defcfun ("pm_priority_max" pm-priority-max)
	  :int)
      
      (defun get-priority-range ()
	(list (priority-min) (priority-max)))
      
      (defparameter *pm-recv-callback*  #'(lambda (x) x (values)))
      
      
      (cffi::defcallback pm_receive_callback :int ((p :pointer))
	(funcall *pm-recv-callback*
		 (cm::pm-message->midi-message (portmidi::Event.message p)))
	0)
      



      #+cm (progn
	     (defmethod cm::recv ((io cm::portmidi-stream) &key (resolution 5) (priority 20))
	       (if (cm::io-open io)
		   (if (= (pm-recv-state) 0)
		       (progn
			 (let ((data (slot-value io 'cm::receive-data)))
			   (if (= (pm-set-input (first data) (second data) (third data)) 0)
			       (if (= (pm-recv-start priority resolution (cffi:callback pm_receive_callback)) 0)
				   (format t "recv running on stream ~s~%"io)))))
		       (warn "recv is already running on ~s" io))
		   (warn "stream ~s is not open" io))
	       (values))
	     
	     (defmethod cm::recv-stop ((io cm::portmidi-stream))
	       (if (cm::io-open io)
		   (if (= (pm-recv-state) 1)
		       (progn
			 (pm-recv-stop)
			 (format t "recv stopped on stream ~s" io))
		       (warn "recv is not running on ~s" io)))
	       (values))
	     
	     (defmethod cm::recv-set! ((io cm::portmidi-stream) hook &key (recv-mode))
	       recv-mode ; make this do something
	       (if (cm::io-open io)
		   (if (functionp hook)
		       (progn
			 (setf *pm-recv-callback* hook))
		       (progn
			 (setf *pm-recv-callback* #'(lambda (x) x (values)))
			 nil))
		   (warn "stream ~s is not open" io))
	       (values))
	     
	     (defmethod cm::recv? ((io cm::portmidi-stream) &optional state)
	       state
	       (if (cm::io-open io)
		   (if (= (pm-recv-state) 0)
		       ':stopped
		       ':running)
		   (warn "stream ~s is not open" io))))))

    
    
